package dev.fyloz.colorrecipesexplorer.model

import org.springframework.security.core.GrantedAuthority
import org.springframework.security.core.authority.SimpleGrantedAuthority

enum class EmployeePermission(
    val impliedPermissions: List<EmployeePermission> = listOf(),
    val deprecated: Boolean = false
) {
    VIEW_RECIPES,
    VIEW_CATALOG,
    VIEW_USERS,

    PRINT_MIXES(listOf(VIEW_RECIPES)),

    EDIT_RECIPES_PUBLIC_DATA(listOf(VIEW_RECIPES)),
    EDIT_RECIPES(listOf(EDIT_RECIPES_PUBLIC_DATA)),
    EDIT_MATERIALS(listOf(VIEW_CATALOG)),
    EDIT_MATERIAL_TYPES(listOf(VIEW_CATALOG)),
    EDIT_COMPANIES(listOf(VIEW_CATALOG)),
    EDIT_USERS(listOf(VIEW_USERS)),
    EDIT_CATALOG(listOf(EDIT_MATERIALS, EDIT_MATERIAL_TYPES, EDIT_COMPANIES)),

    ADD_TO_INVENTORY(listOf(VIEW_CATALOG)),
    DEDUCT_FROM_INVENTORY(listOf(VIEW_RECIPES)),

    REMOVE_RECIPES(listOf(EDIT_RECIPES)),
    REMOVE_MATERIALS(listOf(EDIT_MATERIALS)),
    REMOVE_MATERIAL_TYPES(listOf(EDIT_MATERIAL_TYPES)),
    REMOVE_COMPANIES(listOf(EDIT_COMPANIES)),
    REMOVE_USERS(listOf(EDIT_USERS)),
    REMOVE_CATALOG(listOf(REMOVE_MATERIALS, REMOVE_MATERIAL_TYPES, REMOVE_COMPANIES)),

    ADMIN(
        listOf(
            EDIT_CATALOG,

            REMOVE_RECIPES,
            REMOVE_USERS,
            REMOVE_CATALOG,

            PRINT_MIXES,
            ADD_TO_INVENTORY,
            DEDUCT_FROM_INVENTORY
        )
    ),

    // deprecated permissions
    VIEW_RECIPE(listOf(VIEW_RECIPES), true),
    VIEW_MATERIAL(listOf(VIEW_CATALOG), true),
    VIEW_MATERIAL_TYPE(listOf(VIEW_CATALOG), true),
    VIEW_COMPANY(listOf(VIEW_CATALOG), true),
    VIEW(listOf(VIEW_RECIPES, VIEW_CATALOG), true),
    VIEW_EMPLOYEE(listOf(VIEW_USERS), true),
    VIEW_EMPLOYEE_GROUP(listOf(VIEW_USERS), true),

    EDIT_RECIPE(listOf(EDIT_RECIPES), true),
    EDIT_MATERIAL(listOf(EDIT_MATERIALS), true),
    EDIT_MATERIAL_TYPE(listOf(EDIT_MATERIAL_TYPES), true),
    EDIT_COMPANY(listOf(EDIT_COMPANIES), true),
    EDIT(listOf(EDIT_RECIPES, EDIT_CATALOG), true),
    EDIT_EMPLOYEE(listOf(EDIT_USERS), true),
    EDIT_EMPLOYEE_PASSWORD(listOf(EDIT_USERS), true),
    EDIT_EMPLOYEE_GROUP(listOf(EDIT_USERS), true),

    REMOVE_RECIPE(listOf(REMOVE_RECIPES), true),
    REMOVE_MATERIAL(listOf(REMOVE_MATERIALS), true),
    REMOVE_MATERIAL_TYPE(listOf(REMOVE_MATERIAL_TYPES), true),
    REMOVE_COMPANY(listOf(REMOVE_COMPANIES), true),
    REMOVE(listOf(REMOVE_RECIPES, REMOVE_CATALOG), true),
    REMOVE_EMPLOYEE(listOf(REMOVE_USERS), true),
    REMOVE_EMPLOYEE_GROUP(listOf(REMOVE_USERS), true),

    SET_BROWSER_DEFAULT_GROUP(listOf(VIEW_USERS), true),
    ;

    operator fun contains(permission: EmployeePermission): Boolean {
        return permission == this || impliedPermissions.any { permission in it }
    }
}

fun EmployeePermission.flat(): Iterable<EmployeePermission> {
    return mutableSetOf(this).apply {
        impliedPermissions.forEach {
            addAll(it.flat())
        }
    }
}

/** Converts the given [EmployeePermission] to a [GrantedAuthority]. */
fun EmployeePermission.toAuthority(): GrantedAuthority {
    return SimpleGrantedAuthority(name)
}
