package dev.fyloz.colorrecipesexplorer.model

import dev.fyloz.colorrecipesexplorer.exception.NotFoundException
import javax.persistence.*

@Entity
@Table(name = "recipe_step")
data class RecipeStep(
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    override val id: Long?,

    val position: Int,

    val message: String
) : Model

// ==== DSL ====
fun recipeStep(
    id: Long? = null,
    position: Int = 0,
    message: String = "message",
    op: RecipeStep.() -> Unit = {}
) = RecipeStep(id, position, message).apply(op)

// ==== Exceptions ====
private const val RECIPE_STEP_NOT_FOUND_EXCEPTION_TITLE = "Recipe step not found"
private const val RECIPE_STEP_EXCEPTION_ERROR_CODE = "recipestep"

fun recipeStepIdNotFoundException(id: Long) =
    NotFoundException(
        RECIPE_STEP_EXCEPTION_ERROR_CODE,
        RECIPE_STEP_NOT_FOUND_EXCEPTION_TITLE,
        "A recipe step with the id $id could not be found",
        id
    )
